﻿;;;-*- Mode: Lisp; Package: "ORIBE" -*-
;;;===============================================================================
;;;09/08/02
;;;             AGENT ORIBE: TASKS (file ORIBE-tasks.lisp)
;;;             Copyright Barthès@UTC, 2006
;;;
;;;=============================================================================== 

#|
ORIBE is the assistant agent of JPB.
It can find email addressses, telephone numbers, or use web services (?)

Its ontology is used to process dialogs, answer requests or trigger actions, i.e.
send messages to other agents.
The current file is its task library. It contains a model of all tasks the agent
can execute, related to questions asked to ORIBE:
   - what is the telephone number of "Trigano"?
   - phone "dbb"?
   - home phone of "Moulin"?
   - extension of DICIT?
   - send message to "gapenne".
   - ...

A task can be coupled with a dialog used to fill the task pattern (T-CONTEXT). The 
mechanism is similar to the ACTION mechanism of the MOSS dialogs.
|#

(in-package :ORIBE)

(defparameter *language* :en)  

(format t "~%;*** Loading ORIBE TASKS, language is: ~S" *language*)

;;;=============================================================================== 
;;;   
;;;                               TASK MODEL
;;;
;;;===============================================================================

;;; As soon as the PA finds a task to perform it puts its model into the 
;;; conversation object. 
;;; The Q-CONTEXT will contain the info corresponding to the task arguments,
;;; filled during the specific conversation attached to the task.

(defconcept "task argument"
  (:doc "A TASK ARGUMENT is an argument (required or optional) needed to complete ~
         the task. Questions are attached to the arguments so that they can be ~
         asked from the master if needed.")
  (:att "arg value")
  (:att "arg name" (:unique))
  (:att "arg type")
  (:att "arg key" (:unique))  ; used in the messages as a prefix
  (:att "arg question")
  )

(defconcept (:en "task index")
  (:doc :en "A TASK INDEX is a property of a task. The index attribute specifies ~
             the term being used, the weight attribute gives its importance for the ~
             task.")
  (:att (:en "index"))
  (:att (:en "weight"))
  )

(definstmethod =summary TASK-INDEX ()
  (list (format nil "~{~A ~}(~S)" (has-index)(car (has-weight)))))

(defconcept "task pattern"
  (:att "pattern")
  (:att "weight")
  )

(defconcept "task"
  (:is-a moss::Q-TASK)
  (:doc "A TASK is a model of task to be performed by the assistant.")
  (:att "task name" (:unique)(:entry))
  (:att "target" (:one-of :object :time :cause :subject :owner))
  (:att "performative" (:one-of :request :assert :command))
  (:att "dialog")
  (:att "where to ask")
  (:att "fail question")
  (:rel "subtask" (:to "task"))
  (:rel "index pattern"(:to "task index"))
  (:rel "required args" (:to "task argument"))
  (:rel "optional args" (:to "task argument"))
  )

(definstmethod =summary TASK ()
  (HAS-TASK-NAME))

;;;=============================================================================== 
;;;   
;;;                               TASK LIBRARY
;;;
;;;=============================================================================== 


;;;=========================================================================== ASK

(deftask "ask agent"
  :doc "Task (inform) to ask another agent, presumably a PA."
  :performative :command
  :dialog _ask-conversation
  :indexes ("ask" .8 "warn" .7 "message for" .7 "tell" .4)
  )

;;;======================================================================= EXPLAIN 

(deftask "define concept"
  :doc "Task (command) to explain the meaning of a concept in the ontology."
  :performative :command
  :dialog _explain-conversation
  :indexes ("explain" .5 "define" .6)
  )

;;;=============================================================== GETTING ADDRESS 

(deftask "get address"
  :doc "Task to obtain someone's addresses."
  :dialog _get-address-conversation
  :indexes ("address" .4 "address of" .5 "can find" .4 "happen" .4
            "to be found" .6 "where" .2)
  )

;;;========================================================== GETTING HOME ADDRESS 
;;; must be updated for MCL

(deftask "get home address"
  :doc "Task to obtain someone's addresses."
  :dialog _get-home-address-conversation
  :indexes ("home address" .7 "private address" .7 "lives" .4
            "live" .4 "where" .2 "residence" .5)
  )

;;;================================================ GETTING DEFINITION INFORMATION

;;;======================================================= GETTING NEWS BY SECTION
;;; news are currntly organized in sections, each section containing a topic

(deftask "get news by subject"
  :doc  "Task to obtain news for in specific section."
  :dialog _get-news-by-section-conversation
  :indexes ("news" .5 "last" .4 "facts" .2 "events" .2 "new" .3
            "section" .5 "related to" .5 "about" .3)             
  )

;;;================================================= GETTING TELEPHONE INFORMATION 

(deftask "get phone number"
  :doc "Task to obtain someone's phone number."
  :dialog _get-phone-conversation
  :indexes ("phone number" .7 "telephone number" .7 "number" .4
            "phone" .5 "telephone" .5 "cell-phone" .7)
  )

;;;========================================================================== HELP 

(deftask "help"
  :doc "Task (command) to print some general help."
  :performative :command
  :dialog _print-help-conversation
  :indexes ("SOS" .6 "help" .7 "lost" .4 "what can you do" .4
          "what services" .4)
  )

(deftask "help"
  :doc "Task (question) to print some general help."
  :dialog _print-help-conversation
  :indexes ("SOS" .6 "help" .7 "lost" .4 "what can you do" .4
          "what services" .4)
  )

;;;======================================================== OBTAINING RECENT NEWS

(deftask "obtain last news"
  :doc "Task to show recent news."
  :dialog _get-last-news-conversation
  :indexes ("news" .51 "last" .41 "facts" .21 "events" .21 "new" .31)   
  )

;;;=================================================================== PROCESS ASK 
;;; Generic task to process ASK messages

(deftask "process ask"
  :doc "Generic task to process an ASK message."
  :performative :command
  :dialog _process-ask-conversation
  :indexes (":process-ask" 1)
  )

;;;================================================================== PROCESS TELL 
;;; Generic task to process TELL messages
;;;   This task displays the content of the message and either keeps it or deletes
;;; it.

(deftask "process tell"
  :doc "Generic task to process a TELL message."
  :performative :command
  :dialog _process-tell-conversation
  :indexes (":process-tell" 1)
  )

;;;=================================================================== PUBLISH NEW

(deftask "publish news"
  :doc "Task to obtain publish a new."
  :performative :command
  :dialog _publish-new-conversation
  :indexes ("publish" .6 "share" .3 "news" .5 "hapened" .3 "fact" .3
            "event" .3 "thing" .2)
  )

;;;===================================================================== SET FONTS 

(deftask "font handler"
  :doc "Task to change the size of the dialog font, increasing it, or returning ~
             to the default value."
  :performative :command
  :dialog _set-font-size-conversation
  :indexes ("fonts" .5 "font" .5 "text" .3 "output" .3 "read" .3
            "letters" .3)
  )

;;;========================================================================== TELL

(deftask "tell agent"
  :doc  "Task (inform) to ask another agent, presumably a PA."
  :performative :command
  :dialog _tell-conversation
  :indexes ("inform" .8 "send message" .8 "tell" .7 "information for" .7)
  )
  
;;;========================================================================= TRACE 

(deftask "set trace"
  :doc "Task to switch on and off the dialog trace."
  :performative :command
  :dialog _trace-conversation
  :indexes ("transition" .5 "transitions" .5 "trace" .5 "verbose" .5)
  )

;;;======================================================================= WHAT-IS 

(deftask "what is"
  :doc "Task to obtain information about a person."
  :dialog _what-is-conversation
  :indexes ("what" .2 "is" .2 "what is" .4 "is what" .4)
  )

;;;======================================================================== WHO-IS 

(deftask "who is"
  :doc "Task to obtain information about a person."
  :dialog _get-person-info-conversation
  :indexes ("who is" .7 "who" .2 "is" .2 "whom" .5)
  )

;;;=============================================================================== 

:EOF