﻿;;;-*- Mode: Lisp; Package: "TYLER" -*-
;;;===============================================================================
;;;10/05/19
;;;             AGENT TYLER: TASKS (file TYLER-tasks.lisp)
;;;             Copyright Barthès@UTC, 2006
;;;
;;;=============================================================================== 

#|
TYLER is the assistant agent of JPB.
It can find email addressses, telephone numbers, or use web services (?)

Its ontology is used to process dialogs, answer requests or trigger actions, i.e.
send messages to other agents.
The current file is its task library. It contains a model of all tasks the agent
can execute, related to questions asked to TYLER:
   - what is the telephone number of "Trigano"?
   - phone "dbb"?
   - home phone of "Moulin"?
   - extension of DICIT?
   - send message to "gapenne".
   - ...

A task can be coupled with a dialog used to fill the task pattern (T-CONTEXT). The 
mechanism is similar to the ACTION mechanism of the MOSS dialogs.

2010
 0519 upgrading to version 8.0
|#

(eval-when (compile load eval)
  (unless (find-package :TYLER) (make-package :TYLER)))

(in-package :TYLER)

(eval-when (compile load eval)
  (use-package :moss)
  (use-package :omas)
  (setq *language* :en)
  )  

(format t "~%;*** Loading TYLER TASKS, language is: ~S" *language*)

;;;=============================================================================== 
;;;   
;;;                               TASK MODEL
;;;
;;;===============================================================================

;;; As soon as the PA finds a task to perform it puts its model into the conversation
;;; object. 
;;; The Q-CONTEXT will contain the info corresponding to the task arguments,
;;; filled during the specific conversation attached to the task.

(defconcept "task argument"
  (:doc "A TASK ARGUMENT is an argument (required or optional) needed to complete ~
         the task. Questions are attached to the arguments so that they can be ~
         asked from the master if needed.")
  (:att "arg value")
  (:att "arg name" (:unique))
  (:att "arg type")
  (:att "arg key" (:unique))  ; used in the messages as a prefix
  (:att "arg question")
  )

(defconcept (:en "task index")
  (:doc :en "A TASK INDEX is a property of a task. The index attribute specifies ~
             the term being used, the weight attribute gives its importance for the ~
             task.")
  (:att (:en "index"))
  (:att (:en "weight"))
  )

(definstmethod =summary TASK-INDEX ()
  (list (has-index)(has-weight)))

(defconcept "task pattern"
  (:att "pattern")
  (:att "weight")
  )

(defconcept "task"
  (:is-a moss::Q-TASK)
  (:doc "A TASK is a model of task to be performed by the assistant.")
  (:att "task name" (:unique)(:entry))
  (:att "target" (:one-of :object :time :cause :subject :owner))
  (:att "performative" (:one-of :request :assert :command))
  (:att "dialog")
  (:att "where to ask")
  (:att "fail question")
  (:rel "subtask" (:to "task"))
  (:rel (:en "index pattern")(:to "task index"))
  (:rel "required args" (:to "task argument"))
  (:rel "optional args" (:to "task argument"))
  )

(definstmethod =summary TASK ()
  (HAS-TASK-NAME))

;;;=============================================================================== 
;;;   
;;;                               TASK LIBRARY
;;;
;;;=============================================================================== 


;;;=========================================================================== ASK

(defindividual "task"
  (:doc :en "Task (inform) to ask another agent, presumably a PA."
        :fr "Tâche (inform) pour demander une information à un autre agent")
  ("task name" "ask agent")
  ("performative" :command)
  ("dialog" _ask-conversation)
  ("index pattern"
   
   (:new "task index" ("index" "ask") ("weight" .8))
   (:new "task index" ("index" "warn") ("weight" .7))
   (:new "task index" ("index" "message for") ("weight" .7))
   (:new "task index" ("index" "tell") ("weight" .4))
   )
  )

;;;======================================================================= EXPLAIN 

(defindividual "task"
  (:doc :en "Task (command) to explain the meaning of a concept in the ontology."
        :fr "Tâche (ordre) donnant la définition d'un concept de l'ontologie.")
  ("task name" "define concept")
  ("performative" :command)
  ("dialog" _explain-conversation)
  ("index pattern"
   (:new "task index" ("index" "explain") ("weight" .5))
   (:new "task index" ("index" "define") ("weight" .6))
   )
  )

;;;=============================================================== GETTING ADDRESS 

(defindividual "task argument"
  ("arg name" "get address data")
  (:doc "The GET-AADDRESS-DATA argument contains a list of words to designate ~
         the owner whose address we want.")
  ("arg question" "What is the name again?")
  ("arg key" :data)
  (:var _get-address-data)
  )

(defindividual "task"
  (:doc :en "Task to obtain someone's addresses."
        :fr "Tâche permettant d'obtenir les adresses de quelqu'un")
  ("task name" "get address")
  ("performative" :request)
  ("dialog" _get-address-conversation)
  ("optional args" _get-address-data)
  ("index pattern"
   (:new "task index" ("index" "address")("weight" .5))
   (:new "task index" ("index" "address of")("weight" .7))
   (:new "task index" ("index" "can find")("weight" .4))
   (:new "task index" ("index" "happen")("weight" .4))
   (:new "task index" ("index" "to be found")("weight" .6))
   (:new "task index" ("index" "where")("weight" .2))
   )
  ("where to ask" :address)
  ("fail question" "Whose address?")
  )

;;;========================================================== GETTING HOME ADDRESS 
;;; must be updated for MCL

(defindividual "task"
  (:doc :en "Task to obtain someone's addresses."
        :fr "Tâche permettant d'obtenir l'adresse personnelle de quelqu'un")
  ("task name" "get home address")
  ("performative" :request)
  ("dialog" _get-home-address-conversation)
  ("index pattern"
   (:new "task index" ("index" "home address")("weight" .7))
   (:new "task index" ("index" "private address")("weight" .7))
   (:new "task index" ("index" "lives")("weight" .4))
   (:new "task index" ("index" "live")("weight" .4))
   (:new "task index" ("index" "where")("weight" .2))
   (:new "task index" ("index" "residence")("weight" .5))
   )
  )

;;;================================================ GETTING DEFINITION INFORMATION

;;;======================================================= GETTING NEWS BY SECTION
;;; news are currntly organized in sections, each section containing a topic

(defindividual "task argument"
  ("arg name" "news section")
  (:doc :en "The NEWS-SECTION argument contains the section for obtaining news.")
  ("arg question" "What is the section?")
  ("arg key" :title)
  (:var _get-news-by-section-news)
  )

(defindividual "task"
  ("task name" "get news by subject")
  (:doc :en "Task to obtain news for in specific section.")
  ("performative" :request)
  ("dialog" _get-news-by-section-conversation)
  ("index pattern"
(:new "task index" ("index" "news") ("weight" .5))
   (:new "task index" ("index" "last") ("weight" .4))
   (:new "task index" ("index" "facts") ("weight" .2))
   (:new "task index" ("index" "events") ("weight" .2))   
   (:new "task index" ("index" "new") ("weight" .3))   
   (:new "task index" ("index" "section") ("weight" .5))         
   (:new "task index" ("index" "related to") ("weight" .5))         
   (:new "task index" ("index" "about") ("weight" .3))                  
   )
  ("where to ask" :news)
  ("fail question" "What topic?")
  )

;;;================================================= GETTING TELEPHONE INFORMATION 

(defindividual "task argument"
  ("arg name" "get phone number data")
  (:doc "The GET-PHONE-NUMBER-DATA argument contains a list of words to designate ~
         the owner of the phone number.")
  ("arg question" "Whose number?")
  ("arg key" :data)
  (:var _get-phone-number-data)
  )

(defindividual "task argument"
  ("arg name" "get phone number who")
  (:doc "The GET-PHONE-NUMBER-WHO argument contains an entry-point to designate ~
         the owner of the phone number.")
  ("arg key" :who)
  (:var _get-phone-number-who)
  )

(defindividual "task"
  (:doc :en "Task to obtain someone's phone number."
        :fr "Tâche permettant d'obtenir un numéro de téléphone.")
  ("task name" "get phone number")
  ("performative" :request)
  ("dialog" _get-phone-conversation)
  ("index pattern"
   (:new "task index" ("index" "phone number")("weight" .7))
   (:new "task index" ("index" "telephone number")("weight" .7))
   (:new "task index" ("index" "number")("weight" .4))
   (:new "task index" ("index" "phone")("weight" .5))
   (:new "task index" ("index" "telephone")("weight" .5))
   (:new "task index" ("index" "cell-phone")("weight" .7))
   )
  ("required args" _get-phone-number-data)
  ("optional args" _get-phone-number-who)
  ("fail question" "Whose telephone number?")
  )

;;;========================================================================== HELP 

(defindividual "task"
  (:doc :en "Task (question) to print some general help."
        :fr "Tâche (question) permettant d'imprimer une information générale.")
  ("task name" "help")
  ("performative" :command)
  ("dialog" _print-help-conversation)
  ("index pattern"
   (:new "task index" ("index" "SOS")("weight" .6))
   (:new "task index" ("index" "help")("weight" .7))
   (:new "task index" ("index" "lost")("weight" .4))
   (:new "task index" ("index" "what can you do")("weight" .4))
   (:new "task index" ("index" "what services")("weight" .4))
   )
  )

(defindividual "task"
  (:doc :en "Task (question) to print some general help."
        :fr "Tâche (question) permettant d'imprimer une information générale.")
  ("task name" "help")
  ("performative" :request)
  ("dialog" _print-help-conversation)
  ("index pattern"
   (:new "task index" ("index" "SOS")("weight" .6))
   (:new "task index" ("index" "help")("weight" .7))
   (:new "task index" ("index" "lost")("weight" .4))
   (:new "task index" ("index" "what can you do")("weight" .4))
   (:new "task index" ("index" "what services")("weight" .4))
   )
  )

;;;======================================================== OBTAINING RECENT NEWS

(defindividual "task"
  ("task name" "obtain last news")
  (:doc :en "Task to show recent news.")
  ("performative" :request)
  ("dialog" _get-last-news-conversation)
  ("index pattern"
   (:new "task index" ("index" "news") ("weight" .51))   
   (:new "task index" ("index" "last") ("weight" .41))
   (:new "task index" ("index" "facts") ("weight" .21))   
   (:new "task index" ("index" "events") ("weight" .21))   
   (:new "task index" ("index" "new") ("weight" .31))   
   )
  )

;;;=================================================================== PROCESS ASK 
;;; Generic task to process ASK messages

(defindividual "task"
  (:doc :en "Generic task to process an ASK message."
        :fr "Tâche permettant de traiter un message de type ASK.")
  ("task name" "process ask")
  ("performative" :command)
  ("dialog" _process-ask-conversation)
  ("index pattern"
   (:new "task index" ("index" ":process-ask") ("weight" 1))
   )
  )

;;;================================================================== PROCESS TELL 
;;; Generic task to process TELL messages
;;;   This task displays the content of the message and either keeps it or deletes
;;; it.

(defindividual "task"
  (:doc :en "Generic task to process a TELL message."
        :fr "Tâche permettant de traiter un message de type TELL.")
  ("task name" "process tell")
  ("performative" :command)
  ("dialog" _process-tell-conversation)
  ("index pattern"
   (:new "task index" ("index" ":process-tell") ("weight" 1))
   )
  )

  ;;;=================================================================== PUBLISH NEW

(defindividual "task argument"
  ("arg name" "publish new title")
  (:doc :en "The PUBLISH-NEW-TITLE argument contains the title of the new.")
  ("arg question" "What is the title ?")
  ("arg key" :title)
  (:var _publish-new-title)
  )
  
(defindividual "task argument"
  ("arg name" "publish new content")
  (:doc :en "The PUBLISH-NEW-CONTENT argument contains the content of the new.")
  ("arg question" "What is the content ?")
  ("arg key" :content)
  (:var _publish-new-content)
  )
  
(defindividual "task"
  ("task name" "publish news")
  (:doc :en "Task to obtain publish a new.")
  ("task name" "publish new")
  ("performative" :command)
  ("dialog" _publish-new-conversation)
  ("required args" _publish-new-title _publish-new-content)
  ("index pattern"
   (:new "task index" ("index" "publish")("weight" .6))
   (:new "task index" ("index" "share")("weight" .3))   
   (:new "task index" ("index" "news") ("weight" .5))   
   (:new "task index" ("index" "hapened") ("weight" .3))
   (:new "task index" ("index" "fact") ("weight" .3))
   (:new "task index" ("index" "event") ("weight" .3))
   (:new "task index" ("index" "thing") ("weight" .2))   
   )
  ("where to ask" :news)
  ("fail question" "What topic?")
  )

;;;===================================================================== SET FONTS 

(defindividual "task argument"
  ("arg name" "dialog font size")
  (:doc "The DIALOG-FONT-SIZE argument contains the requested size of the text ~
         used in the interwaction with the user.")
  ("arg question" "What size again?")
  ("arg key" :size)
  (:var _dialog-font-size)
  )

(defindividual "task"
  (:doc :en "Task to change the size of the dialog font, increasing it, or returning ~
             to the default value."
        :fr "Tâche permettant de modifier la taille des caractères sur l'écran.")
  ("task name" "font handler")
  ("performative" :command)
  ("dialog" _set-font-size-conversation)
  ("index pattern"
   (:new "task index" ("index" "fonts") ("weight" .5))
   (:new "task index" ("index" "font")("weight" .5))
   (:new "task index" ("index" "text")("weight" .3))
   (:new "task index" ("index" "output")("weight" .3))
   (:new "task index" ("index" "read")("weight" .3))
   (:new "task index" ("index" "letters")("weight" .3))
   )
  ("optional args" _dialog-font-size)
  ("fail question" "Do you want a bigger or smaller text size?")
  )

;;;========================================================================== TELL

(defindividual "task"
  (:doc :en "Task (inform) to ask another agent, presumably a PA.")
  ("task name" "tell agent")
  ("performative" :command)
  ("dialog" _tell-conversation)
  ("index pattern"
   (:new "task index" ("index" "inform") ("weight" .8))
   (:new "task index" ("index" "send message") ("weight" .8))
   (:new "task index" ("index" "tell") ("weight" .7))
   (:new "task index" ("index" "information for") ("weight" .7))
   )
  )
;;;========================================================================= TRACE 

(defindividual "task"
  (:doc :en "Task to switch on and off the dialog trace."
        :fr "Tâche permettant d'activer ou de supprimer la trace.")
  ("task name" "set trace")
  ("performative" :command)
  ("dialog" _trace-conversation)
  ("index pattern"
   (:new "task index" ("index" "transition") ("weight" .5))
   (:new "task index" ("index" "transitions")("weight" .5))
   (:new "task index" ("index" "trace")("weight" .5))
   (:new "task index" ("index" "verbose")("weight" .5))
   )
  )

;;;======================================================================= WHAT-IS 

(defindividual "task"
  (:doc :en "Task to obtain information about a person."
        :fr "Tâche permettant d'obtenir des infos sur une expression.")
  ("task name" "what is")
  ("performative" :request)
  ("dialog" _what-is-conversation)
  ("index pattern"
   (:new "task index" ("index" "what")("weight" .2))
   (:new "task index" ("index" "is")("weight" .2))
   (:new "task index" ("index" "what is")("weight" .4))
   (:new "task index" ("index" "is what")("weight" .4))
   )
  )

;;;======================================================================== WHO-IS 

(defindividual "task"
  (:doc :en "Task to obtain information about a person."
        :fr "Tâche permettant d'obtenir des infos sur une expression.")
  ("task name" "who is")
  ("performative" :request)
  ("dialog" _get-person-info-conversation)
  ("index pattern"
   (:new "task index" ("index" "who is")("weight" .7))
   (:new "task index" ("index" "who")("weight" .2))
   (:new "task index" ("index" "is")("weight" .2))
   (:new "task index" ("index" "whom")("weight" .5))
   )
  )

;;;=============================================================================== 

:EOF