/**
 * Set of functions for graph rendering 
 * Author: Marcio Fuckner
 *
 * Revision history:
 * 05/11/2013 - MFK - Bug correction allowing several divs exhibition.
 */

// global variables 
var colors = [], // array of available node colors
    shapes = [], // array of available shape types
	conceptStyles = {},  // array of styles
	colorPos, // last color used
	shapePos, // last shape used
	sequence = 0; // last object sequence

/**
 * Render a graph in the specified div
 * Note: API dependent function (Infovis Toolkit)
 * Parameters:
 *   obj - the json representation of the graph
 *   labelFunctions - the set of label functions
 *   divName - the target div
 */ 

function renderGraph(obj,labelFunctions,divName){

  var nodes = {};

  initColors();
  initShapes();

  conceptStyles = {};	
  
  sequence ++;
  
  $jit.RGraph.Plot.EdgeTypes.implement({
    'labeled': {
    'render': function(adj, canvas) {
                this.edgeTypes.line.render.call(this, adj, canvas);
                var data = adj.data;
                if(data.relation && data.relation != "result") {
                  var ctx = canvas.getCtx();
                  var posFr = adj.nodeFrom.pos.getc(true);
                  var posTo = adj.nodeTo.pos.getc(true);
                  ctx.fillText(data.relation, 
				    (posFr.x + posTo.x)/2, (posFr.y + posTo.y)/2);
                }
              }
  }});
  
  // label function
  var createLabelFunction = function(domElement, node){
      // get label function for the corresponding concept type
      labelFunction = getLabelFunction(node.data.type,labelFunctions);
		
      var label = labelFunction(node);
				
      domElement.innerHTML = label;
      domElement.onclick = function () {
  	    rgraph.onClick(node.id, { 
          hideLabels: false,
          onComplete: function() {
		    // none
          }
        });
      };
	  
      var style = domElement.style;
          style.cursor = 'pointer';
          style.fontSize = "0.8em";
          style.color = "#606060";
  };
  
  //RGraph constructor.
  var rgraph = new $jit.RGraph({
    'injectInto': divName,
    // canvas circles
	'background': {
      'CanvasStyles': {
        'strokeStyle': '#c0c0c0',
        'shadowBlur': 100,
        'shadowColor': '#ffffff',
      }
    },
    Navigation: {  
      enable: true,  
      panning: true,
      zooming: 10 //zoom speed. higher is more sensible  
    },
    Node: {
      'overridable': true,
      'dim': 15,
    },
    Edge: {
      'overridable': true,
      'color': '#cccc00',
      'type': 'labeled'
    },
    Events: {
      enable: true,
      onMouseEnter: function (node, eventInfo) {
        if (node) {		 
          showDetails(node,eventInfo.getPos());
        }
      },
      onMouseLeave: function() {
        hideDetails();
      },
      // click handler to node
      onClick: function(node) {
        // none
      }
    },
		
    // Set polar interpolation.
    // Default's linear.
    interpolation: 'polar',
	
    // Change the transition effect from linear
    // to elastic.
    transition: $jit.Trans.Elastic.easeOut,
    
    //Change other animation parameters.
    duration:2000,
    fps: 30,
    //Change father-child distance.
    levelDistance: 150,

	// This method is called right before plotting
    // an edge. This method is useful to change edge styles
    // individually.
    onBeforePlotLine: function(adj){
      // none
    },
    onBeforeCompute: function(node){
      // none
    },
    
    // Add node click handler and some styles.
    // This method is called only once for each node/label is created.
    onCreateLabel: createLabelFunction,
    
	// This method is called when rendering/moving a label.
    // This is method is useful to make some last minute changes
    // to node labels like adding some position offset.
    onPlaceLabel: function(domElement, node) {
		
      var style = domElement.style;
          style.display = '';
          style.cursor = 'pointer';

      if (node._depth <= 1) {
        style.fontSize = "0.8em";
        style.color = "#000000";
      } else if(node._depth == 2){
        style.fontSize = "0.7em";
        style.color = "#000000";
      } else {
        style.fontSize = "0.7em";
        style.color = "#494949";
      }

      var left = parseInt(style.left);
      var w = domElement.offsetWidth;
      style.left = (left - w / 2) + 'px';
    }
  });
  	
  // create one root object containing the result
  var rootObj = createRootObject(obj);
	
  // create nodes	
  createNode(rootObj,nodes);
  
  // add nodes to the graph tool	
  addNodes(nodes, rgraph);

  //compute positions and plot
  rgraph.refresh();		
}

/**
 * Used to look for the label function to be used on the node
 * Note: API independent function
 * Parameters:
 *    type: The concept type (e.g.: person, project).
 *    labelFunctions: the set of label functions
 * Returns:
 *   the label function.
 */
function getLabelFunction(type,labelFunctions) {
	var labelFunction = labelFunctions[type];
	if(!labelFunction)
		labelFunction = function(node) { return node.id };
	return labelFunction;
}

/**
 * Create node relations
 * Note: API dependent function (Infovis Toolkit)
 * Parameters:
 *   relations: object containing parent and childs
 *   rgraph: the graph tool object
 * Returns:
 *   none
 */
function addRelations(relations,rgraph) {
	for(var i=0;i<relations.length;i++) {
		rgraph.graph.addAdjacence(relations[i].parent,relations[i].child,relations[i].relation);
	}
}

/**
 * Add nodes to the graph tool
 *  Note: API dependent function (Infovis Toolkit)
 *  Parameters:
 *    nodes: the set of nodes
 *    rgraph: the graph tool object
 * Returns:
 *   none
 */
function addNodes(nodes,rgraph) {
	for(var node in nodes) {
	
		nodeStyle = getStyle(nodes[node].data.type);
		nodes[node].data.$color = nodeStyle.$color;
		nodes[node].data.$type = nodeStyle.$type;
		
		rgraph.graph.addNode(nodes[node]);
		if(nodes[node].id.substring(0,1) == "0") {
			rgraph.root = nodes[node].id;
		}
		for(var i=0;i<nodes[node].relations.length;i++) {
			rgraph.graph.addAdjacence(nodes[node].relations[i].parent,
										nodes[node].relations[i].child,
										{"relation":nodes[node].relations[i].relation,
										 "labelid" : nodes[node].relations[i].parent.id + "_" +
										             nodes[node].relations[i].child.id});
		}
	}
}

/**
 * Show node details 
 * Note: API dependent function (Infovis Toolkit)
 * Parameters:
 *   node: the node object
 *   pos: node position
 */
function showDetails(node,pos) {
  var e = window.event;
  
  html = "<ul>";
  for(var elem in node.data) {
    if(elem.substring(0,1) == "$")
      continue;				   
    html += "<li>" + elem + " = " + node.data[elem] + "</li>"
  }
  html += "</ul>";
  
  var div = document.getElementById('inner-details');
  div.style.position = 'absolute';

  // Set the div top to the height 
  div.style.top = (event.clientY + 50) + "px";

  // Set the div Left to the height 
  div.style.left = (event.clientX + 50) + "px";
  div.style.width = "300px";
     
  // Show the div layer
  div.style.display = 'block';
  div.innerHTML = html;
}

/**
 * Hide node details 
 * Note: API independent function
 */
function hideDetails() {
  var div = document.getElementById('inner-details');
   div.style.display = 'none';
}

/**
 * Create root node
 * Note: API independent function
 * Parameters:
 *   source: object containing the details of the node
 */
function createRootObject(source) {
	var root = {"id" : "0" + "_" + sequence, "result" : source, "type" : "result"};
	return root;
}

/**
 * Create node
 * Note: API independent function
 * Parameters:
 *   obj: source object
 *   nodes: set of created nodes
 */
function createNode(obj,nodes) {
	var node = {}, data = {};		

	if(obj == null)
	   return null;
	   
	node.relations = [];

	var prop;
	for(prop in obj) {
		if(typeof obj[prop] == 'string') {
			data[prop] = obj[prop];
		} else if(obj[prop] instanceof Array) {
			var arrStr = [];
			var aux = obj[prop];
			for(var i=0;i<aux.length;i++) {
				if(typeof aux[i] == 'string') {
					arrStr.push(aux[i]);
				} else if(typeof aux[i] == 'object' && aux[i] != null) {
					var child = createNode(aux[i],nodes);
					var relation = {"parent" : node, "child" : child, "relation" : prop};
					node.relations.push(relation);
				}					
			}
			if(arrStr.length != 0)
				data[prop] = arrStr;
		} else if(typeof obj[prop] == 'object' && obj[prop] != null) {
			var child = createNode(obj[prop],nodes);
			var relation = {"parent" : node, "child" : child, "relation" : prop};
			node.relations.push(relation);
		}
	}
	node.data = data;	
	
	if(!node.id)
		node.id = data.id[0] + "_" + sequence;
				
	nodes[node.id] = node;
	return node;
}

/**
 * Initialize the set of colors
 * Note: API independent function
 */
function initColors() {
    colors = [];
    colors.push("#00FF00");
    colors.push("#00FFFF");
    colors.push("#FF00FF");
    colors.push("#FFFF00");
    colors.push("#C0C0C0");
    colorPos = 0;
}

/**
 * get next available color
 */
function getNextColor() {
    return colors[colorPos ++ % colors.length];
}

/**
 * Initialize the set of shapes
 * Note: API independent function
 */
function initShapes() {
  shapes = [];
  shapes.push("circle");
  shapes.push("square");
  shapes.push("triangle");
  shapePos = 0;
}

/**
 * get next available shape
 */
function getNextShape() {
	return shapes[shapePos ++ % shapes.length];
}

/**
 * get next available style
 */
function getStyle(type) {
	
	if(!conceptStyles[type]) {
		conceptStyles[type] = {"$color" : getNextColor() , "$type" : getNextShape()};
	}
	return conceptStyles[type];
}

/* EOF */